#!/usr/bin/perl

use strict;
use File::Spec::Functions qw(rel2abs);
use File::Basename;
my $ToolChainDir = dirname(rel2abs($0));

my $ProjectFolder = shift(@ARGV);

my $ScriptName="activate_project.sh";

unless ($ProjectFolder) {
  print "$0 PROJECT_FOLDER\n";
  print "Creates a new activate-script in given project folder.\n";
  print "\n";
  exit 5;
}


my $AllActivateScripts = `ls $ENV{HOME}/Source/ToolChain_STM32/ToolChain/extensions/activate.*`;
my @AllActivateScripts = split("\n", $AllActivateScripts);
@AllActivateScripts = map { my @Parts = split("/", $_); $Parts[-1]; } @AllActivateScripts;

my %Ranks;
foreach my $ActivateScript (@AllActivateScripts) {
  (my $Rank, my @Rest) = split("_", $ActivateScript);
  my $DotPos = rindex($Rank, '.');
  $Rank = substr($Rank, $DotPos+1);
  unless ($Ranks{$Rank}) {
    $Ranks{$Rank} = { 
      Scripts => [ ],        # all scripts being found in $HOME/Source/ToolChain_STM32/ToolChain/extensions/
      Comment => '',         # placed at beginning of each rank-block
      DefaultScripts => []   # list of scripts being printed as non-commented
    };
    #print "new rank: '$Rank'\n"; 
  }
  push @{ $Ranks{$Rank}->{Scripts} }, $ActivateScript;
  $Ranks{$Rank}->{DefaultScripts}->[0] = $ActivateScript;
}

#{ set comments + defaults
my $Rank;

$Rank = '000'; $Ranks{$Rank}->{Name}="activate basic setup"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
END_OF_COMMENT
$Ranks{$Rank}->{DefaultScripts} = ['activate.000_basic_extensions_basic_setup.sh'];
#}000
$Rank = '050'; $Ranks{$Rank}->{Name}="compiler settings"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
END_OF_COMMENT
#}050
$Rank = '100'; $Ranks{$Rank}->{Name}="activate your board"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # Each board 
  # - activates a CPU
  # - sets memory sizes and locations
  # - activates required standard peripheral library (STM-types only)
  #
  # In case your board is not listed, create a copy of the nearest
  # ~/Source/ToolChain_STM32/ToolChain/install_.sh to create an extension for your board.

END_OF_COMMENT
#}100
$Rank = '200'; $Ranks{$Rank}->{Name}="CPU"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # The CPU should already been activated by the board makefile.
  # You may want to activate a CPU in case you don't yet have a board makefile 
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}
$Rank = '250'; $Ranks{$Rank}->{Name}="Std Peripherals Library"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  # board has activated CPU + Std Peripherals Library
  # activate certain .c-files of Std Peripherals Library
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}250
$Rank = '270'; $Ranks{$Rank}->{Name}="Communication Application Peripheral Library"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  # CPAL has been designed on top of std peripherals library
  # activate certain .c-files of CPAL
  #
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}270
$Rank = '300'; $Ranks{$Rank}->{Name}="multitasking scheduler"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # A multitasking scheduler provides basic multithreading features.
  # The activation of one scheduler and a heap type is mandatory. 
  #
END_OF_COMMENT
push @{ $Ranks{$Rank}->{DefaultScripts} }, 'activate.300_scheduler_free_rtos_heap1.sh';
#}
$Rank = '400'; $Ranks{$Rank}->{Name}="External Support Libraries"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}
$Rank = '500'; $Ranks{$Rank}->{Name}="Support Libraries provided by ToolChain_STM32"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # These libraries provide example code for how to initialize
  # and use certain funtional units like watchdogs, adcs, ...
  #
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}
$Rank = '600'; $Ranks{$Rank}->{Name}="Examples"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # Only one example can be active at a time.
  # In order to run, each example requires
  # - board (must define PB_LED1, PB_LED2 and others)
  # - cpu
  # - multitasking scheduler
  # - standard peripheral library
  #
  # 600_example_leds is the most simple example and should work on any board that defines PB_LED1, PB_LED2
  #
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = ['activate.600_example_leds.sh'];
#}
$Rank = ''; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 

END_OF_COMMENT
#}
#}

my @Activates;
foreach my $Rank (sort keys %Ranks) { # compile list of all available activates
  if ($Rank) {
    my $RankRef = $Ranks{$Rank};
    my $Activate = $RankRef->{Comment};
    foreach my $ActivateScript (@{ $RankRef->{Scripts} }) {
      my $Info = `$ActivateScript INFO`;
      my @Info = split("\n", $Info);
      @Info = grep { $_ } @Info; # remove empty lines
      if (@Info) { $Activate .= "\n  # ".join("\n  # ", @Info)."\n"; }
      $Activate .= "  #$ActivateScript QUIET\n";
    }
    foreach my $DefaultScripts (@{ $RankRef->{DefaultScripts} }) {
      $Activate .= "  $DefaultScripts QUIET\n";
    }
    push @Activates, "#{ $Rank - ".$Ranks{$Rank}->{Name}."\n".$Activate."\n#}$Rank";
  }
}
my $Activates = join("\n", @Activates);

my $Script = <<"END_OF_SCRIPT"; #{ compile content of activate-script
#!/bin/bash

ExtensionsActive=`ls extensions.active/makefile.*`
if [ "\$ExtensionsActive" == ""  ]; then
  echo "\$0 - no extensions active: activating basic setup"

  # golden rule: start to activate lower ranks first:
  # activate.000_XXX.sh
  # activate.100_XXX.sh
  # ...

$Activates

  # this makefile belongs to current project and is added automatically
  cd extensions.active/
  
  # activate following lines to use your own board file in extensions.local/
  # rm ../extensions.local/makefile.100_board_*
  # ln -s makefile.100_board_* ../extensions.local/
  
  # activate local makefile
  ln -s ../extensions.local/makefile.700_extra_settings .
  
  cd ..
else
  echo "\$0 - extension setup found: not changing extensions.active/"
fi
END_OF_SCRIPT
#}

if (-e $ScriptName) { system("mv $ScriptName $ScriptName.old"); }
open(OUT, ">$ScriptName") or die("$0 - ERROR: Cannot create activate-script `pwd`/$ScriptName ($!)");
print OUT $Script;
close(OUT) or die("$0 - ERROR: Cannot write to activate-script `pwd`/$ScriptName ($!)");
system("chmod +x \"$ScriptName\"");
